@@ -13,15 +13,19 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+ * MA 02110-1301, USA.
  */
 
 #include <Python.h>
 #include "structmember.h"
+#include <sys/ioctl.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <fcntl.h>
+#include <linux/i2c.h>
 #include <linux/i2c-dev.h>
+#include <i2c/smbus.h>
 
 /*
 ** These are required to build this module against Linux older than 2.6.23.
@@ -34,9 +38,10 @@
 
 PyDoc_STRVAR(SMBus_module_doc,
 	"This module defines an object type that allows SMBus transactions\n"
-	"on Linux hosts.  The host kernel must have I2C support, I2C device\n"
-	"interface support, and a bus adapter driver.  All of these can be\n"
-	"either built-in to the kernel, or loaded from modules.\n"
+	"on hosts running the Linux kernel.  The host kernel must have I2C\n"
+	"support, I2C device interface support, and a bus adapter driver.\n"
+	"All of these can be either built-in to the kernel, or loaded from\n"
+	"modules.\n"
 	"\n"
 	"Because the I2C device interface is opened R/W, users of this\n"
 	"module usually must have root permissions.\n");
@@ -90,7 +95,11 @@ SMBus_dealloc(SMBus *self)
 	PyObject *ref = SMBus_close(self);
 	Py_XDECREF(ref);
 
+#if PY_MAJOR_VERSION >= 3
+	Py_TYPE(self)->tp_free((PyObject *)self);
+#else
 	self->ob_type->tp_free((PyObject *)self);
+#endif
 }
 
 #define MAXPATH 16
@@ -207,7 +216,7 @@ SMBus_read_byte(SMBus *self, PyObject *args)
 
 	SMBus_SET_ADDR(self, addr);
 
-	if ((result = i2c_smbus_read_byte(self->fd)) == -1) {
+	if ((result = i2c_smbus_read_byte(self->fd)) < 0) {
 		PyErr_SetFromErrno(PyExc_IOError);
 		return NULL;
 	}
@@ -230,7 +239,7 @@ SMBus_write_byte(SMBus *self, PyObject *args)
 
 	SMBus_SET_ADDR(self, addr);
 
-	if ((result = i2c_smbus_write_byte(self->fd, (__u8)val)) == -1) {
+	if ((result = i2c_smbus_write_byte(self->fd, (__u8)val)) < 0) {
 		PyErr_SetFromErrno(PyExc_IOError);
 		return NULL;
 	}
@@ -254,7 +263,7 @@ SMBus_read_byte_data(SMBus *self, PyObject *args)
 
 	SMBus_SET_ADDR(self, addr);
 
-	if ((result = i2c_smbus_read_byte_data(self->fd, (__u8)cmd)) == -1) {
+	if ((result = i2c_smbus_read_byte_data(self->fd, (__u8)cmd)) < 0) {
 		PyErr_SetFromErrno(PyExc_IOError);
 		return NULL;
 	}
@@ -278,7 +287,7 @@ SMBus_write_byte_data(SMBus *self, PyObject *args)
 	SMBus_SET_ADDR(self, addr);
 
 	if ((result = i2c_smbus_write_byte_data(self->fd,
-				(__u8)cmd, (__u8)val)) == -1) {
+				(__u8)cmd, (__u8)val)) < 0) {
 		PyErr_SetFromErrno(PyExc_IOError);
 		return NULL;
 	}
@@ -302,7 +311,7 @@ SMBus_read_word_data(SMBus *self, PyObject *args)
 
 	SMBus_SET_ADDR(self, addr);
 
-	if ((result = i2c_smbus_read_word_data(self->fd, (__u8)cmd)) == -1) {
+	if ((result = i2c_smbus_read_word_data(self->fd, (__u8)cmd)) < 0) {
 		PyErr_SetFromErrno(PyExc_IOError);
 		return NULL;
 	}
@@ -326,7 +335,7 @@ SMBus_write_word_data(SMBus *self, PyObject *args)
 	SMBus_SET_ADDR(self, addr);
 
 	if ((result = i2c_smbus_write_word_data(self->fd,
-				(__u8)cmd, (__u16)val)) == -1) {
+				(__u8)cmd, (__u16)val)) < 0) {
 		PyErr_SetFromErrno(PyExc_IOError);
 		return NULL;
 	}
@@ -351,7 +360,7 @@ SMBus_process_call(SMBus *self, PyObject *args)
 	SMBus_SET_ADDR(self, addr);
 
 	if ((result = i2c_smbus_process_call(self->fd,
-				(__u8)cmd, (__u16)val)) == -1) {
+				(__u8)cmd, (__u16)val)) < 0) {
 		PyErr_SetFromErrno(PyExc_IOError);
 		return NULL;
 	}
@@ -430,11 +439,19 @@ SMBus_list_to_data(PyObject *list, union i2c_smbus_data *data)
 
 	for (ii = 0; ii < len; ii++) {
 		PyObject *val = PyList_GET_ITEM(list, ii);
+#if PY_MAJOR_VERSION >= 3
+		if (!PyLong_Check(val)) {
+			PyErr_SetString(PyExc_TypeError, msg);
+			return 0; /* fail */
+		}
+		data->block[ii+1] = (__u8)PyLong_AS_LONG(val);
+#else
 		if (!PyInt_Check(val)) {
 			PyErr_SetString(PyExc_TypeError, msg);
 			return 0; /* fail */
 		}
 		data->block[ii+1] = (__u8)PyInt_AS_LONG(val);
+#endif
 	}
 
 	return 1; /* success */
@@ -633,9 +650,14 @@ static PyGetSetDef SMBus_getset[] = {
 };
 
 static PyTypeObject SMBus_type = {
+#if PY_MAJOR_VERSION >= 3
+	PyVarObject_HEAD_INIT(NULL, 0)
+	"SMBus",			/* tp_name */
+#else
 	PyObject_HEAD_INIT(NULL)
 	0,				/* ob_size */
-	"SMBus",			/* tp_name */
+	"smbus.SMBus",			/* tp_name */
+#endif
 	sizeof(SMBus),			/* tp_basicsize */
 	0,				/* tp_itemsize */
 	(destructor)SMBus_dealloc,	/* tp_dealloc */
@@ -674,24 +696,50 @@ static PyTypeObject SMBus_type = {
 	SMBus_new,			/* tp_new */
 };
 
+#if PY_MAJOR_VERSION >= 3
+static struct PyModuleDef SMBusModule = {
+	PyModuleDef_HEAD_INIT,
+	"SMBus",			/* m_name */
+	SMBus_module_doc,  		/* m_doc */
+	-1,				/* m_size */
+	NULL,				/* m_methods */
+	NULL,				/* m_reload */
+	NULL,				/* m_traverse */
+	NULL,				/* m_clear */
+	NULL,				/* m_free */
+};
+#define INIT_RETURN(m)	return m
+#define INIT_FNAME	PyInit_smbus
+#else
 static PyMethodDef SMBus_module_methods[] = {
 	{NULL}
 };
+#define INIT_RETURN(m)	return
+#define INIT_FNAME	initsmbus
+#endif
 
 #ifndef PyMODINIT_FUNC	/* declarations for DLL import/export */
 #define PyMODINIT_FUNC void
 #endif
 PyMODINIT_FUNC
-initsmbus(void) 
+INIT_FNAME(void)
 {
 	PyObject* m;
 
 	if (PyType_Ready(&SMBus_type) < 0)
-		return;
+		INIT_RETURN(NULL);
 
+#if PY_MAJOR_VERSION >= 3
+	m = PyModule_Create(&SMBusModule);
+#else
 	m = Py_InitModule3("smbus", SMBus_module_methods, SMBus_module_doc);
+#endif
+	if (m == NULL)
+		INIT_RETURN(NULL);
 
 	Py_INCREF(&SMBus_type);
 	PyModule_AddObject(m, "SMBus", (PyObject *)&SMBus_type);
+
+	INIT_RETURN(m);
 }
 
